@@ -290,7 +290,11 @@ static bool set_cpu_affinity(logical_cpu_t logical_cpu)
 
 static int get_total_cpus(void)
 {
+#ifdef HW_NCPUONLINE
+	int mib[2] = { CTL_HW, HW_NCPUONLINE };
+#else
 	int mib[2] = { CTL_HW, HW_NCPU };
+#endif
 	int ncpus;
 	size_t len = sizeof(ncpus);
 	if (sysctl(mib, 2, &ncpus, &len, (void *) 0, 0) != 0) return 1;
@@ -713,6 +717,8 @@ static int cpuid_serialize_raw_data_internal(struct cpu_raw_data_t* single_raw,
 					fprintf(f, "arm_id_aa64afr%d=%016" PRIx64 "\n", i, raw_ptr->arm_id_aa64afr[i]);
 				for (i = 0; i < MAX_ARM_ID_AA64DFR_REGS; i++)
 					fprintf(f, "arm_id_aa64dfr%d=%016" PRIx64 "\n", i, raw_ptr->arm_id_aa64dfr[i]);
+				for (i = 0; i < MAX_ARM_ID_AA64FPFR_REGS; i++)
+					fprintf(f, "arm_id_aa64fpfr%d=%016" PRIx64 "\n", i, raw_ptr->arm_id_aa64fpfr[i]);
 				for (i = 0; i < MAX_ARM_ID_AA64ISAR_REGS; i++)
 					fprintf(f, "arm_id_aa64isar%d=%016" PRIx64 "\n", i, raw_ptr->arm_id_aa64isar[i]);
 				for (i = 0; i < MAX_ARM_ID_AA64MMFR_REGS; i++)
@@ -751,7 +757,7 @@ static int cpuid_deserialize_raw_data_internal(struct cpu_raw_data_t* single_raw
 	bool is_libcpuid_dump = true;
 	bool is_aida64_dump = false;
 	const bool use_raw_array = (raw_array != NULL);
-	logical_cpu_t logical_cpu = 0;
+	logical_cpu_t logical_cpu = 0, logical_cpu_offset = 0;
 	uint32_t addr, eax, ebx, ecx, edx, aarch32_reg;
 	uint64_t aarch64_reg;
 	char version[8] = "";
@@ -798,6 +804,7 @@ static int cpuid_deserialize_raw_data_internal(struct cpu_raw_data_t* single_raw
 			else if (!strcmp(line, "------[ Versions ]------") ||
 			         !strcmp(line, "------[ Logical CPU #0 ]------") ||
 			         !strcmp(line, "------[ CPUID Registers / Logical CPU #0 ]------") ||
+			         !strcmp(line, "CPUID Registers (CPU #1):") ||
 			         strstr(line, "CPU#000 AffMask: 0x")) {
 				debugf(2, "Recognized AIDA64 raw dump\n");
 				is_header = false;
@@ -868,6 +875,9 @@ static int cpuid_deserialize_raw_data_internal(struct cpu_raw_data_t* single_raw
 			else if ((sscanf(line, "arm_id_aa64dfr%d=%" SCNx64, &i, &aarch64_reg) >= 2)) {
 				RAW_ASSIGN_LINE_AARCH64(raw_ptr->arm_id_aa64dfr[i]);
 			}
+			else if ((sscanf(line, "arm_id_aa64fpfr%d=%" SCNx64, &i, &aarch64_reg) >= 2)) {
+				RAW_ASSIGN_LINE_AARCH64(raw_ptr->arm_id_aa64fpfr[i]);
+			}
 			else if ((sscanf(line, "arm_id_aa64isar%d=%" SCNx64, &i, &aarch64_reg) >= 2)) {
 				RAW_ASSIGN_LINE_AARCH64(raw_ptr->arm_id_aa64isar[i]);
 			}
@@ -890,7 +900,12 @@ static int cpuid_deserialize_raw_data_internal(struct cpu_raw_data_t* single_raw
 		else if (is_aida64_dump) {
 			if (use_raw_array && ((sscanf(line, "------[ Logical CPU #%" SCNu16 " ]------", &logical_cpu) >= 1) ||
 			                      (sscanf(line, "------[ CPUID Registers / Logical CPU #%" SCNu16 " ]------", &logical_cpu) >= 1) ||
+			                      (sscanf(line, "CPUID Registers (CPU #%" SCNu16, &logical_cpu) >= 1) ||
 			                      (sscanf(line, "CPU#%" SCNu16 " AffMask: 0x%*x", &logical_cpu) >= 1))) {
+				/* Some raw dumps start core count from 1, we need to start from 0 */
+				if ((raw_array->num_raw == 0) && (logical_cpu >= 1))
+					logical_cpu_offset = logical_cpu;
+				logical_cpu -= logical_cpu_offset;
 				debugf(2, "Parsing AIDA64 raw dump for logical CPU %i\n", logical_cpu);
 				cpuid_grow_raw_data_array(raw_array, logical_cpu + 1);
 				raw_ptr = &raw_array->raw[logical_cpu];
@@ -899,6 +914,8 @@ static int cpuid_deserialize_raw_data_internal(struct cpu_raw_data_t* single_raw
 			}
 			subleaf = 0;
 			assigned = sscanf(line, "CPUID %" SCNx32 ": %" SCNx32 "-%" SCNx32 "-%" SCNx32 "-%" SCNx32 " [SL %02i]", &addr, &eax, &ebx, &ecx, &edx, &subleaf);
+			if (assigned == 1)
+				assigned = sscanf(line, "CPUID %" SCNx32 "  	 %" SCNx32 "-%" SCNx32 "-%" SCNx32 "-%" SCNx32 " [SL %02i]", &addr, &eax, &ebx, &ecx, &edx, &subleaf);
 			debugf(3, "raw line %d: %i items assigned for string '%s'\n", cur_line, assigned, line);
 			if ((assigned >= 5) && (subleaf == 0)) {
 				if (addr < MAX_CPUID_LEVEL) {
@@ -928,7 +945,7 @@ static int cpuid_deserialize_raw_data_internal(struct cpu_raw_data_t* single_raw
 	/* Close file descriptor */
 	if (strcmp(filename, ""))
 		fclose(f);
-	return cpuid_set_error(ERR_OK);
+	return cpuid_set_error((use_raw_array && (raw_array->num_raw == 0)) ? ERR_BADFMT : ERR_OK);
 }
 #undef RAW_ASSIGN_LINE_X86
 #undef RAW_ASSIGN_LINE_ARM
@@ -1414,6 +1431,8 @@ int cpuid_get_raw_data_core(struct cpu_raw_data_t* data, logical_cpu_t logical_c
 			cpu_read_arm_register_64b(handle, REQ_ID_AA64AFR0 + i, &data->arm_id_aa64afr[i]);
 		for (i = 0; i < MAX_ARM_ID_AA64DFR_REGS; i++)
 			cpu_read_arm_register_64b(handle, REQ_ID_AA64DFR0 + i, &data->arm_id_aa64dfr[i]);
+		for (i = 0; i < MAX_ARM_ID_AA64FPFR_REGS; i++)
+			cpu_read_arm_register_64b(handle, REQ_ID_AA64FPFR0 + i, &data->arm_id_aa64fpfr[i]);
 		for (i = 0; i < MAX_ARM_ID_AA64ISAR_REGS; i++)
 			cpu_read_arm_register_64b(handle, REQ_ID_AA64ISAR0 + i, &data->arm_id_aa64isar[i]);
 		for (i = 0; i < MAX_ARM_ID_AA64MMFR_REGS; i++)
@@ -1440,9 +1459,12 @@ int cpuid_get_raw_data_core(struct cpu_raw_data_t* data, logical_cpu_t logical_c
 		cpu_exec_mrs(AARCH64_REG_ID_AA64AFR1_EL1, data->arm_id_aa64afr[1]);
 		cpu_exec_mrs(AARCH64_REG_ID_AA64DFR0_EL1, data->arm_id_aa64dfr[0]);
 		cpu_exec_mrs(AARCH64_REG_ID_AA64DFR1_EL1, data->arm_id_aa64dfr[1]);
+		cpu_exec_mrs(AARCH64_REG_ID_AA64DFR2_EL1, data->arm_id_aa64dfr[2]);
+		cpu_exec_mrs(AARCH64_REG_ID_AA64FPFR0_EL1, data->arm_id_aa64fpfr[0]);
 		cpu_exec_mrs(AARCH64_REG_ID_AA64ISAR0_EL1, data->arm_id_aa64isar[0]);
 		cpu_exec_mrs(AARCH64_REG_ID_AA64ISAR1_EL1, data->arm_id_aa64isar[1]);
 		cpu_exec_mrs(AARCH64_REG_ID_AA64ISAR2_EL1, data->arm_id_aa64isar[2]);
+		cpu_exec_mrs(AARCH64_REG_ID_AA64ISAR3_EL1, data->arm_id_aa64isar[3]);
 		cpu_exec_mrs(AARCH64_REG_ID_AA64MMFR0_EL1, data->arm_id_aa64mmfr[0]);
 		cpu_exec_mrs(AARCH64_REG_ID_AA64MMFR1_EL1, data->arm_id_aa64mmfr[1]);
 		cpu_exec_mrs(AARCH64_REG_ID_AA64MMFR2_EL1, data->arm_id_aa64mmfr[2]);
@@ -1810,7 +1832,7 @@ const char* cpu_architecture_str(cpu_architecture_t architecture)
 {
 	const struct { cpu_architecture_t architecture; const char* name; }
 	matchtable[] = {
-		{ ARCHITECTURE_UNKNOWN, "unknown" },
+		{ ARCHITECTURE_UNKNOWN, UNKN_STR  },
 		{ ARCHITECTURE_X86,     "x86"     },
 		{ ARCHITECTURE_ARM,     "ARM"     },
 	};
@@ -1828,7 +1850,7 @@ const char* cpu_feature_level_str(cpu_feature_level_t level)
 {
 	const struct { cpu_feature_level_t level; const char* name; }
 	matchtable[] = {
-		{ FEATURE_LEVEL_UNKNOWN,   "unknown"   },
+		{ FEATURE_LEVEL_UNKNOWN,   UNKN_STR    },
 		/* x86 */
 		{ FEATURE_LEVEL_I386,      "i386"      },
 		{ FEATURE_LEVEL_I486,      "i486"      },
@@ -1872,6 +1894,7 @@ const char* cpu_feature_level_str(cpu_feature_level_t level)
 		{ FEATURE_LEVEL_ARM_V9_2_A, "ARMv9.2-A" },
 		{ FEATURE_LEVEL_ARM_V9_3_A, "ARMv9.3-A" },
 		{ FEATURE_LEVEL_ARM_V9_4_A, "ARMv9.4-A" },
+		{ FEATURE_LEVEL_ARM_V9_5_A, "ARMv9.5-A" },
 	};
 	unsigned i, n = COUNT_OF(matchtable);
 	if (n != (NUM_FEATURE_LEVELS - FEATURE_LEVEL_ARM_V1) + (FEATURE_LEVEL_X86_64_V4 - FEATURE_LEVEL_I386) + 2) {
@@ -2049,14 +2072,17 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_AVX512VBMI, "avx512vbmi" },
 		{ CPU_FEATURE_AVX512VBMI2, "avx512vbmi2" },
 		{ CPU_FEATURE_HYPERVISOR, "hypervisor" },
+		/* Arm */
 		{ CPU_FEATURE_SWAP, "swap" },
 		{ CPU_FEATURE_THUMB, "thumb" },
 		{ CPU_FEATURE_ADVMULTU, "advmultu" },
 		{ CPU_FEATURE_ADVMULTS, "advmults" },
 		{ CPU_FEATURE_JAZELLE, "jazelle" },
+		/* Armv6.0 */
 		{ CPU_FEATURE_DEBUGV6, "debugv6" },
 		{ CPU_FEATURE_DEBUGV6P1, "debugv6p1" },
 		{ CPU_FEATURE_THUMB2, "thumb2" },
+		/* Armv7.0 */
 		{ CPU_FEATURE_DEBUGV7, "debugv7" },
 		{ CPU_FEATURE_DEBUGV7P1, "debugv7p1" },
 		{ CPU_FEATURE_THUMBEE, "thumbee" },
@@ -2064,6 +2090,7 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_LPAE, "lpae" },
 		{ CPU_FEATURE_PMUV1, "pmuv1" },
 		{ CPU_FEATURE_PMUV2, "pmuv2" },
+		/* A2.2.1 The Armv8.0 architecture extension */
 		{ CPU_FEATURE_ASID16, "asid16" },
 		{ CPU_FEATURE_ADVSIMD, "advsimd" },
 		{ CPU_FEATURE_CRC32, "crc32" },
@@ -2081,6 +2108,7 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_SHA1, "sha1" },
 		{ CPU_FEATURE_SHA256, "sha256" },
 		{ CPU_FEATURE_NTLBPA, "ntlbpa" },
+		/* A2.2.2 The Armv8.1 architecture extension */
 		{ CPU_FEATURE_HAFDBS, "hafdbs" },
 		{ CPU_FEATURE_HPDS, "hpds" },
 		{ CPU_FEATURE_LOR, "lor" },
@@ -2090,6 +2118,7 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_RDM, "rdm" },
 		{ CPU_FEATURE_VHE, "vhe" },
 		{ CPU_FEATURE_VMID16, "vmid16" },
+		/* A2.2.3 The Armv8.2 architecture extension */
 		{ CPU_FEATURE_AA32HPD, "aa32hpd" },
 		{ CPU_FEATURE_AA32I8MM, "aa32i8mm" },
 		{ CPU_FEATURE_DPB, "dpb" },
@@ -2114,6 +2143,7 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_TTCNP, "ttcnp" },
 		{ CPU_FEATURE_UAO, "uao" },
 		{ CPU_FEATURE_XNX, "xnx" },
+		/* A2.2.4 The Armv8.3 architecture extension */
 		{ CPU_FEATURE_CCIDX, "ccidx" },
 		{ CPU_FEATURE_CONSTPACFIELD, "constpacfield" },
 		{ CPU_FEATURE_EPAC, "epac" },
@@ -2127,6 +2157,7 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_PACQARMA5, "pacqarma5" },
 		{ CPU_FEATURE_PAUTH, "pauth" },
 		{ CPU_FEATURE_SPEV1P1, "spev1p1" },
+		/* A2.2.5 The Armv8.4 architecture extension */
 		{ CPU_FEATURE_AMUV1, "amuv1" },
 		{ CPU_FEATURE_BBM, "bbm" },
 		{ CPU_FEATURE_DIT, "dit" },
@@ -2148,6 +2179,7 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_TRF, "trf" },
 		{ CPU_FEATURE_TTL, "ttl" },
 		{ CPU_FEATURE_TTST, "ttst" },
+		/* A2.2.6 The Armv8.5 architecture extension */
 		{ CPU_FEATURE_BTI, "bti" },
 		{ CPU_FEATURE_CSV2, "csv2" },
 		{ CPU_FEATURE_CSV3, "csv3" },
@@ -2166,6 +2198,7 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_SPECRES, "specres" },
 		{ CPU_FEATURE_SSBS, "ssbs" },
 		{ CPU_FEATURE_SSBS2, "ssbs2" },
+		/* A2.2.7 The Armv8.6 architecture extension */
 		{ CPU_FEATURE_AA32BF16, "aa32bf16" },
 		{ CPU_FEATURE_AMUV1P1, "amuv1p1" },
 		{ CPU_FEATURE_BF16, "bf16" },
@@ -2178,6 +2211,7 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_MTPMU, "mtpmu" },
 		{ CPU_FEATURE_PAUTH2, "pauth2" },
 		{ CPU_FEATURE_TWED, "twed" },
+		/* A2.2.8 The Armv8.7 architecture extension */
 		{ CPU_FEATURE_AFP, "afp" },
 		{ CPU_FEATURE_EBF16, "ebf16" },
 		{ CPU_FEATURE_HCX, "hcx" },
@@ -2193,6 +2227,7 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_SPEV1P2, "spev1p2" },
 		{ CPU_FEATURE_WFXT, "wfxt" },
 		{ CPU_FEATURE_XS, "xs" },
+		/* A2.2.9 The Armv8.8 architecture extension */
 		{ CPU_FEATURE_CMOW, "cmow" },
 		{ CPU_FEATURE_DEBUGV8P8, "debugv8p8" },
 		{ CPU_FEATURE_HBC, "hbc" },
@@ -2203,6 +2238,7 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_SPEV1P3, "spev1p3" },
 		{ CPU_FEATURE_TCR2, "tcr2" },
 		{ CPU_FEATURE_TIDCP1, "tidcp1" },
+		/* A2.2.10 The Armv8.9 architecture extension */
 		{ CPU_FEATURE_ADERR, "aderr" },
 		{ CPU_FEATURE_AIE, "aie" },
 		{ CPU_FEATURE_ANERR, "anerr" },
@@ -2238,6 +2274,7 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_SPEV1P4, "spev1p4" },
 		{ CPU_FEATURE_SPMU, "spmu" },
 		{ CPU_FEATURE_THE, "the" },
+		/* A2.3.1 The Armv9.0 architecture extension */
 		{ CPU_FEATURE_SVE2, "sve2" },
 		{ CPU_FEATURE_SVE_AES, "sve_aes" },
 		{ CPU_FEATURE_SVE_BITPERM, "sve_bitperm" },
@@ -2246,15 +2283,18 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_SVE_SM4, "sve_sm4" },
 		{ CPU_FEATURE_TME, "tme" },
 		{ CPU_FEATURE_TRBE, "trbe" },
+		/* A2.3.3 The Armv9.2 architecture extension */
 		{ CPU_FEATURE_BRBE, "brbe" },
 		{ CPU_FEATURE_RME, "rme" },
 		{ CPU_FEATURE_SME, "sme" },
 		{ CPU_FEATURE_SME_F64F64, "sme_f64f64" },
 		{ CPU_FEATURE_SME_FA64, "sme_fa64" },
 		{ CPU_FEATURE_SME_I16I64, "sme_i16i64" },
+		/* A2.3.4 The Armv9.3 architecture extension */
 		{ CPU_FEATURE_BRBEV1P1, "brbev1p1" },
 		{ CPU_FEATURE_MEC, "mec" },
 		{ CPU_FEATURE_SME2, "sme2" },
+		/* A2.3.5 The Armv9.4 architecture extension */
 		{ CPU_FEATURE_ABLE, "able" },
 		{ CPU_FEATURE_BWE, "bwe" },
 		{ CPU_FEATURE_D128, "d128" },
@@ -2271,6 +2311,35 @@ const char* cpu_feature_str(cpu_feature_t feature)
 		{ CPU_FEATURE_SYSINSTR128, "sysinstr128" },
 		{ CPU_FEATURE_SYSREG128, "sysreg128" },
 		{ CPU_FEATURE_TRBE_EXT, "trbe_ext" },
+		/* A2.3.6 The Armv9.5 architecture extension */
+		{ CPU_FEATURE_ASID2, "asid2" },
+		{ CPU_FEATURE_BWE2, "bwe2" },
+		{ CPU_FEATURE_CPA, "cpa" },
+		{ CPU_FEATURE_CPA2, "cpa2" },
+		{ CPU_FEATURE_E2H0, "e2h0" },
+		{ CPU_FEATURE_E3DSE, "e3dse" },
+		{ CPU_FEATURE_ETS3, "ets3" },
+		{ CPU_FEATURE_FAMINMAX, "faminmax" },
+		{ CPU_FEATURE_FGWTE3, "fgwte3" },
+		{ CPU_FEATURE_FP8, "fp8" },
+		{ CPU_FEATURE_FP8DOT2, "fp8dot2" },
+		{ CPU_FEATURE_FP8DOT4, "fp8dot4" },
+		{ CPU_FEATURE_FP8FMA, "fp8fma" },
+		{ CPU_FEATURE_FPMR, "fpmr" },
+		{ CPU_FEATURE_HACDBS, "hacdbs" },
+		{ CPU_FEATURE_HDBSS, "hdbss" },
+		{ CPU_FEATURE_LUT, "lut" },
+		{ CPU_FEATURE_PAUTH_LR, "pauth_lr" },
+		{ CPU_FEATURE_RME_GPC2, "rme_gpc2" },
+		{ CPU_FEATURE_SME_F8F16, "sme_f8f16" },
+		{ CPU_FEATURE_SME_F8F32, "sme_f8f32" },
+		{ CPU_FEATURE_SME_LUTV2, "sme_lutv2" },
+		{ CPU_FEATURE_SPMU2, "spmu2" },
+		{ CPU_FEATURE_SSVE_FP8DOT2, "ssve_fp8dot2" },
+		{ CPU_FEATURE_SSVE_FP8DOT4, "ssve_fp8dot4" },
+		{ CPU_FEATURE_SSVE_FP8FMA, "ssve_fp8fma" },
+		{ CPU_FEATURE_STEP2, "step2" },
+		{ CPU_FEATURE_TLBIW, "tlbiw" },
 	};
 	unsigned i, n = COUNT_OF(matchtable);
 	if (n != NUM_CPU_FEATURES) {
