@@ -96,6 +96,7 @@ struct msr_driver_t* cpu_msr_driver_open_core(unsigned core_num)
 	handle = (struct msr_driver_t*) malloc(sizeof(struct msr_driver_t));
 	if (!handle) {
 		cpuid_set_error(ERR_NO_MEM);
+		close(fd);
 		return NULL;
 	}
 	handle->fd = fd;
@@ -490,33 +491,47 @@ struct msr_driver_t* cpu_msr_driver_open(void)
 
 struct msr_driver_t* cpu_msr_driver_open_core(unsigned core_num)
 {
+	UNUSED(core_num);
 	cpuid_set_error(ERR_NOT_IMP);
 	return NULL;
 }
 
 int cpu_rdmsr(struct msr_driver_t* driver, uint32_t msr_index, uint64_t* result)
 {
+	UNUSED(driver);
+	UNUSED(msr_index);
+	UNUSED(result);
 	return cpuid_set_error(ERR_NOT_IMP);
 }
 
 int cpu_msr_driver_close(struct msr_driver_t* driver)
 {
+	UNUSED(driver);
 	return cpuid_set_error(ERR_NOT_IMP);
 }
 
 int cpu_rdmsr_range(struct msr_driver_t* handle, uint32_t msr_index, uint8_t highbit,
                     uint8_t lowbit, uint64_t* result)
 {
+	UNUSED(handle);
+	UNUSED(msr_index);
+	UNUSED(highbit);
+	UNUSED(lowbit);
+	UNUSED(result);
 	return cpuid_set_error(ERR_NOT_IMP);
 }
 
 int cpu_msrinfo(struct msr_driver_t* driver, cpu_msrinfo_request_t which)
 {
+	UNUSED(driver);
+	UNUSED(which);
 	return cpuid_set_error(ERR_NOT_IMP);
 }
 
 int msr_serialize_raw_data(struct msr_driver_t* handle, const char* filename)
 {
+	UNUSED(handle);
+	UNUSED(filename);
 	return cpuid_set_error(ERR_NOT_IMP);
 }
 
@@ -645,6 +660,201 @@ static int perfmsr_measure(struct msr_driver_t* handle, int msr)
 	return (int) ((y - x) / (b - a));
 }
 
+static int msr_platform_info_supported(struct msr_info_t *info)
+{
+	int i;
+	static int supported = -1;
+
+	/* Return cached result */
+	if(supported >= 0)
+		return supported;
+
+	/* List of microarchitectures that provide both "Maximum Non-Turbo Ratio" and "Maximum Efficiency Ratio" values
+	Please note Silvermont does not report "Maximum Efficiency Ratio" */
+	const struct { int32_t ext_family; int32_t ext_model; } msr_platform_info[] = {
+		/* Table 2-12. MSRs in Intel Atom® Processors Based on Goldmont Microarchitecture */
+		{ 6, 92 },
+		{ 6, 122 },
+		/* Table 2-15. MSRs in Processors Based on Nehalem Microarchitecture */
+		{ 6, 26 },
+		{ 6, 30 },
+		{ 6, 37 },
+		{ 6, 44 },
+		/* Table 2-20. MSRs Supported by Intel® Processors Based on Sandy Bridge Microarchitecture */
+		{ 6, 42 },
+		{ 6, 45 },
+		/* Table 2-25. Additional MSRs Supported by 3rd Generation Intel® CoreTM Processors Based on Ivy Bridge Microarchitecture */
+		{ 6, 58 },
+		/* Table 2-26. MSRs Supported by the Intel® Xeon® Processor E5 v2 Product Family (Ivy Bridge-E Microarchitecture) */
+		{ 6, 62 },
+		/* Table 2-29. Additional MSRs Supported by Processors Based on the Haswell and Haswell-E Microarchitectures */
+		{ 6, 60 },
+		{ 6, 63 },
+		{ 6, 69 },
+		{ 6, 70 },
+		/* Table 2-36. Additional MSRs Common to the Intel® Xeon® Processor D and the Intel® Xeon® Processor E5 v4 Family Based on Broadwell Microarchitecture */
+		{ 6, 61 },
+		{ 6, 71 },
+		{ 6, 79 },
+		/* Table 2-39. Additional MSRs Supported by the 6th—13th Generation Intel® CoreTM Processors, 1st—5th Generation Intel® Xeon® Scalable Processor Families, Intel® CoreTM Ultra 7 Processors, 8th Generation Intel® CoreTM i3 Processors, and Intel® Xeon® E Processors */
+		/* ==> Skylake */
+		{ 6, 78 },
+		{ 6, 85 },
+		{ 6, 94 },
+		/* ==> Kaby Lake */
+		{ 6, 142 },
+		{ 6, 158 },
+		/* ==> Coffee Lake */
+		{ 6, 102 },
+		{ 6, 142 },
+		{ 6, 158 },
+		/* ==> Cascade Lake */
+		{ 6, 85 },
+		/* ==> Comet Lake */
+		{ 6, 142 },
+		{ 6, 165 },
+		/* ==> Ice Lake */
+		{ 6, 106 },
+		{ 6, 108 },
+		{ 6, 126 },
+		/* ==> Rocket Lake */
+		{ 6, 167 },
+		/* ==> Tremont */
+		{ 6, 138 },
+		{ 6, 150 },
+		{ 6, 156 },
+		/* ==> Tiger Lake */
+		{ 6, 140 },
+		/* ==> Alder Lake */
+		{ 6, 151 },
+		{ 6, 154 },
+		{ 6, 190 },
+		/* ==> Raptor Lake */
+		{ 6, 183 },
+		{ 6, 186 },
+		{ 6, 191 },
+		/* ==> Sapphire Rapids */
+		{ 6, 143 },
+		/* ==> Emerald Rapids */
+		{ 6, 207 },
+		/* ==> Meteor Lake */
+		{ 6, 170 },
+		/* ==> Arrow Lake */
+		{ 6, 198 },
+		/* ==> Lunar Lake */
+		{ 6, 189 },
+		/* Table 2-50. MSRs Supported by the Intel® Xeon® Scalable Processor Family with a CPUID Signature DisplayFamily_DisplayModel Value of 06_55H */
+		{ 0x6, 0x55 },
+		/* Table 2-56. Selected MSRs Supported by Intel® Xeon PhiTM Processors with a CPUID Signature DisplayFamily_DisplayModel Value of 06_57H or 06_85H */
+		{ 0x6, 0x57 },
+		{ 0x6, 0x85 },
+	};
+
+	if(info->id->vendor == VENDOR_INTEL) {
+		for(i = 0; i < COUNT_OF(msr_platform_info); i++) {
+			if((info->id->x86.ext_family == msr_platform_info[i].ext_family) && (info->id->x86.ext_model == msr_platform_info[i].ext_model)) {
+				debugf(2, "Intel CPU with CPUID signature %02X_%02XH supports MSR_PLATFORM_INFO.\n", info->id->x86.ext_family, info->id->x86.ext_model);
+				supported = 1;
+				return supported;
+			}
+		}
+		debugf(2, "Intel CPU with CPUID signature %02X_%02XH does not support MSR_PLATFORM_INFO.\n", info->id->x86.ext_family, info->id->x86.ext_model);
+	}
+
+	supported = 0;
+	return supported;
+}
+
+static int msr_intel_core_supported(struct msr_info_t *info)
+{
+	int i;
+	static int supported = -1;
+
+	/* Return cached result */
+	if(supported >= 0)
+		return supported;
+
+	/* List of microarchitectures that provide "Core Voltage" values */
+	const struct { int32_t ext_family; int32_t ext_model; } msr_perf_status[] = {
+		/* Table 2-20. MSRs Supported by Intel® Processors Based on Sandy Bridge Microarchitecture */
+		{ 6, 42 },
+		{ 6, 45 },
+		/* ==> Ivy Bridge */
+		{ 6, 58 },
+		{ 6, 62 },
+		/* ==> Haswell */
+		{ 6, 60 },
+		{ 6, 63 },
+		{ 6, 69 },
+		{ 6, 70 },
+		/* ==> Broadwell */
+		{ 6, 61 },
+		{ 6, 71 },
+		{ 6, 79 },
+		/* ==> Skylake */
+		{ 6, 78 },
+		{ 6, 85 },
+		{ 6, 94 },
+		/* ==> Kaby Lake */
+		{ 6, 142 },
+		{ 6, 158 },
+		/* ==> Coffee Lake */
+		{ 6, 102 },
+		{ 6, 142 },
+		{ 6, 158 },
+		/* ==> Cascade Lake */
+		{ 6, 85 },
+		/* ==> Comet Lake */
+		{ 6, 142 },
+		{ 6, 165 },
+		/* ==> Ice Lake */
+		{ 6, 106 },
+		{ 6, 108 },
+		{ 6, 126 },
+		/* ==> Rocket Lake */
+		{ 6, 167 },
+		/* ==> Tremont */
+		{ 6, 138 },
+		{ 6, 150 },
+		{ 6, 156 },
+		/* ==> Tiger Lake */
+		{ 6, 140 },
+		/* ==> Alder Lake */
+		{ 6, 151 },
+		{ 6, 154 },
+		{ 6, 190 },
+		/* ==> Raptor Lake */
+		{ 6, 183 },
+		{ 6, 186 },
+		{ 6, 191 },
+		/* ==> Sapphire Rapids */
+		{ 6, 143 },
+		/* ==> Emerald Rapids */
+		{ 6, 207 },
+		/* ==> Meteor Lake */
+		{ 6, 170 },
+		/* Table 2-50. MSRs Supported by the Intel® Xeon® Scalable Processor Family with a CPUID Signature DisplayFamily_DisplayModel Value of 06_55H */
+		{ 0x6, 0x55 },
+		/* Table 2-56. Selected MSRs Supported by Intel® Xeon PhiTM Processors with a CPUID Signature DisplayFamily_DisplayModel Value of 06_57H or 06_85H */
+		{ 0x6, 0x57 },
+		{ 0x6, 0x85 },
+	};
+
+	if(info->id->vendor == VENDOR_INTEL) {
+		for(i = 0; i < COUNT_OF(msr_perf_status); i++) {
+			if((info->id->x86.ext_family == msr_perf_status[i].ext_family) && (info->id->x86.ext_model == msr_perf_status[i].ext_model)) {
+				debugf(2, "Intel CPU with CPUID signature %02X_%02XH supports MSR_PERF_STATUS.\n", info->id->x86.ext_family, info->id->x86.ext_model);
+				supported = 1;
+				return supported;
+			}
+		}
+		debugf(2, "Intel CPU with CPUID signature %02X_%02XH does not support MSR_PERF_STATUS.\n", info->id->x86.ext_family, info->id->x86.ext_model);
+	}
+
+	supported = 0;
+	return supported;
+}
+
 static int get_amd_multipliers(struct msr_info_t *info, uint32_t pstate, double *multiplier)
 {
 	int i, err;
@@ -665,8 +875,8 @@ static int get_amd_multipliers(struct msr_info_t *info, uint32_t pstate, double
 	const int num_dids = (int) COUNT_OF(divisor_t);
 
 	/* Constant values for common families */
-	const int magic_constant = (info->id->ext_family == 0x11) ? 0x8 : 0x10;
-	const int is_apu = ((FUSION_C <= info->internal->code.amd) && (info->internal->code.amd <= FUSION_A)) || (info->internal->bits & _APU_);
+	const int magic_constant = (info->id->x86.ext_family == 0x11) ? 0x8 : 0x10;
+	const int is_apu = (strstr(info->id->brand_str, "APU") != NULL) || (strstr(info->id->brand_str, "Radeon ") != NULL);
 	const double divisor = is_apu ? 1.0 : 2.0;
 
 	/* Check if P-state is valid */
@@ -674,7 +884,7 @@ static int get_amd_multipliers(struct msr_info_t *info, uint32_t pstate, double
 		return 1;
 
 	/* Overview of AMD CPU microarchitectures: https://en.wikipedia.org/wiki/List_of_AMD_CPU_microarchitectures#Nomenclature */
-	switch (info->id->ext_family) {
+	switch (info->id->x86.ext_family) {
 		case 0x12: /* K10 (Llano) / K12 */
 			/* BKDG 12h, page 469
 			MSRC001_00[6B:64][8:4] is CpuFid
@@ -754,7 +964,7 @@ static int get_amd_multipliers(struct msr_info_t *info, uint32_t pstate, double
 			*multiplier = ((double) CpuFid / CpuDid) * 2;
 			break;
 		default:
-			warnf("get_amd_multipliers(): unsupported CPU extended family: %xh\n", info->id->ext_family);
+			warnf("get_amd_multipliers(): unsupported CPU extended family: %xh\n", info->id->x86.ext_family);
 			err = 1;
 			break;
 	}
@@ -789,7 +999,7 @@ static double get_info_min_multiplier(struct msr_info_t *info)
 	uint32_t addr;
 	uint64_t reg;
 
-	if(info->id->vendor == VENDOR_INTEL) {
+	if(msr_platform_info_supported(info)) {
 		/* Refer links above
 		Table 35-12.  MSRs in Next Generation Intel Atom Processors Based on the Goldmont Microarchitecture
 		Table 35-13.  MSRs in Processors Based on Intel® Microarchitecture Code Name Nehalem
@@ -821,17 +1031,19 @@ static double get_info_cur_multiplier(struct msr_info_t *info)
 	double mult;
 	uint64_t reg;
 
-	if(info->id->vendor == VENDOR_INTEL && info->internal->code.intel == PENTIUM) {
-		err = cpu_rdmsr(info->handle, MSR_EBL_CR_POWERON, &reg);
-		if (!err) return (double) ((reg>>22) & 0x1f);
-	}
-	else if(info->id->vendor == VENDOR_INTEL && info->internal->code.intel != PENTIUM) {
-		/* Refer links above
-		Table 35-2.  IA-32 Architectural MSRs (Contd.)
-		IA32_PERF_STATUS[15:0] is Current performance State Value
-		[7:0] is 0x0, [15:8] looks like current ratio */
-		err = cpu_rdmsr_range(info->handle, IA32_PERF_STATUS, 15, 8, &reg);
-		if (!err) return (double) reg;
+	if(info->id->vendor == VENDOR_INTEL) {
+		if(!msr_intel_core_supported(info)) {
+			err = cpu_rdmsr(info->handle, MSR_EBL_CR_POWERON, &reg);
+			if (!err) return (double) ((reg>>22) & 0x1f);
+		}
+		else {
+			/* Refer links above
+			Table 35-2.  IA-32 Architectural MSRs (Contd.)
+			IA32_PERF_STATUS[15:0] is Current performance State Value
+			[7:0] is 0x0, [15:8] looks like current ratio */
+			err = cpu_rdmsr_range(info->handle, IA32_PERF_STATUS, 15, 8, &reg);
+			if (!err) return (double) reg;
+		}
 	}
 	else if(info->id->vendor == VENDOR_AMD || info->id->vendor == VENDOR_HYGON) {
 		/* Refer links above
@@ -850,28 +1062,30 @@ static double get_info_max_multiplier(struct msr_info_t *info)
 	double mult;
 	uint64_t reg;
 
-	if(info->id->vendor == VENDOR_INTEL && info->internal->code.intel == PENTIUM) {
-		err = cpu_rdmsr(info->handle, IA32_PERF_STATUS, &reg);
-		if (!err) return (double) ((reg >> 40) & 0x1f);
-	}
-	else if(info->id->vendor == VENDOR_INTEL && info->internal->code.intel != PENTIUM) {
-		/* Refer links above
-		Table 35-10.  Specific MSRs Supported by Intel® Atom™ Processor C2000 Series with CPUID Signature 06_4DH
-		Table 35-12.  MSRs in Next Generation Intel Atom Processors Based on the Goldmont Microarchitecture (Contd.)
-		Table 35-13.  MSRs in Processors Based on Intel® Microarchitecture Code Name Nehalem (Contd.)
-		Table 35-14.  Additional MSRs in Intel® Xeon® Processor 5500 and 3400 Series
-		Table 35-16.  Additional MSRs Supported by Intel Processors (Based on Intel® Microarchitecture Code Name Westmere)
-		Table 35-19.  MSRs Supported by 2nd Generation Intel® Core™ Processors (Intel® microarchitecture code name Sandy Bridge)
-		Table 35-21.  Selected MSRs Supported by Intel® Xeon® Processors E5 Family (based on Sandy Bridge microarchitecture)
-		Table 35-28.  MSRs Supported by 4th Generation Intel® Core™ Processors (Haswell microarchitecture) (Contd.)
-		Table 35-30.  Additional MSRs Supported by Intel® Xeon® Processor E5 v3 Family
-		Table 35-33.  Additional MSRs Supported by Intel® Core™ M Processors and 5th Generation Intel® Core™ Processors
-		Table 35-34.  Additional MSRs Common to Intel® Xeon® Processor D and Intel Xeon Processors E5 v4 Family Based on the Broadwell Microarchitecture
-		Table 35-37.  Additional MSRs Supported by 6th Generation Intel® Core™ Processors Based on Skylake Microarchitecture
-		Table 35-40.  Selected MSRs Supported by Next Generation Intel® Xeon Phi™ Processors with DisplayFamily_DisplayModel Signature 06_57H
-		MSR_TURBO_RATIO_LIMIT[7:0] is Maximum Ratio Limit for 1C */
-		err = cpu_rdmsr_range(info->handle, MSR_TURBO_RATIO_LIMIT, 7, 0, &reg);
-		if (!err) return (double) reg;
+	if(info->id->vendor == VENDOR_INTEL) {
+		if(!msr_intel_core_supported(info)) {
+			err = cpu_rdmsr(info->handle, IA32_PERF_STATUS, &reg);
+			if (!err) return (double) ((reg >> 40) & 0x1f);
+		}
+		else {
+			/* Refer links above
+			Table 35-10.  Specific MSRs Supported by Intel® Atom™ Processor C2000 Series with CPUID Signature 06_4DH
+			Table 35-12.  MSRs in Next Generation Intel Atom Processors Based on the Goldmont Microarchitecture (Contd.)
+			Table 35-13.  MSRs in Processors Based on Intel® Microarchitecture Code Name Nehalem (Contd.)
+			Table 35-14.  Additional MSRs in Intel® Xeon® Processor 5500 and 3400 Series
+			Table 35-16.  Additional MSRs Supported by Intel Processors (Based on Intel® Microarchitecture Code Name Westmere)
+			Table 35-19.  MSRs Supported by 2nd Generation Intel® Core™ Processors (Intel® microarchitecture code name Sandy Bridge)
+			Table 35-21.  Selected MSRs Supported by Intel® Xeon® Processors E5 Family (based on Sandy Bridge microarchitecture)
+			Table 35-28.  MSRs Supported by 4th Generation Intel® Core™ Processors (Haswell microarchitecture) (Contd.)
+			Table 35-30.  Additional MSRs Supported by Intel® Xeon® Processor E5 v3 Family
+			Table 35-33.  Additional MSRs Supported by Intel® Core™ M Processors and 5th Generation Intel® Core™ Processors
+			Table 35-34.  Additional MSRs Common to Intel® Xeon® Processor D and Intel Xeon Processors E5 v4 Family Based on the Broadwell Microarchitecture
+			Table 35-37.  Additional MSRs Supported by 6th Generation Intel® Core™ Processors Based on Skylake Microarchitecture
+			Table 35-40.  Selected MSRs Supported by Next Generation Intel® Xeon Phi™ Processors with DisplayFamily_DisplayModel Signature 06_57H
+			MSR_TURBO_RATIO_LIMIT[7:0] is Maximum Ratio Limit for 1C */
+			err = cpu_rdmsr_range(info->handle, MSR_TURBO_RATIO_LIMIT, 7, 0, &reg);
+			if (!err) return (double) reg;
+		}
 	}
 	else if(info->id->vendor == VENDOR_AMD || info->id->vendor == VENDOR_HYGON) {
 		/* Refer links above
@@ -889,7 +1103,7 @@ static int get_info_temperature(struct msr_info_t *info)
 	int err;
 	uint64_t DigitalReadout, ReadingValid, TemperatureTarget;
 
-	if(info->id->vendor == VENDOR_INTEL) {
+	if(msr_intel_core_supported(info)) {
 		/* Refer links above
 		Table 35-2.   IA-32 Architectural MSRs
 		IA32_THERM_STATUS[22:16] is Digital Readout
@@ -917,7 +1131,7 @@ static double get_info_voltage(struct msr_info_t *info)
 	double VIDStep;
 	uint64_t reg, CpuVid;
 
-	if(info->id->vendor == VENDOR_INTEL) {
+	if(msr_intel_core_supported(info)) {
 		/* Refer links above
 		Table 35-18.  MSRs Supported by Intel® Processors based on Intel® microarchitecture code name Sandy Bridge (Contd.)
 		MSR_PERF_STATUS[47:32] is Core Voltage
@@ -933,9 +1147,9 @@ static double get_info_voltage(struct msr_info_t *info)
 		BKDG 10h, page 49: voltage = 1.550V - 0.0125V * SviVid (SVI1)
 		BKDG 15h, page 50: Voltage = 1.5500 - 0.00625 * Vid[7:0] (SVI2)
 		SVI2 since Piledriver (Family 15h, 2nd-gen): Models 10h-1Fh Processors */
-		VIDStep = ((info->id->ext_family < 0x15) || ((info->id->ext_family == 0x15) && (info->id->ext_model < 0x10))) ? 0.0125 : 0.00625;
+		VIDStep = ((info->id->x86.ext_family < 0x15) || ((info->id->x86.ext_family == 0x15) && (info->id->x86.ext_model < 0x10))) ? 0.0125 : 0.00625;
 		err = cpu_rdmsr_range(info->handle, MSR_PSTATE_S, 2, 0, &reg);
-		if(info->id->ext_family < 0x17)
+		if(info->id->x86.ext_family < 0x17)
 			err += cpu_rdmsr_range(info->handle, MSR_PSTATE_0 + (uint32_t) reg, 15, 9, &CpuVid);
 		else
 			err += cpu_rdmsr_range(info->handle, MSR_PSTATE_0 + (uint32_t) reg, 21, 14, &CpuVid);
@@ -952,7 +1166,7 @@ static double get_info_bus_clock(struct msr_info_t *info)
 	uint32_t addr;
 	uint64_t reg;
 
-	if(info->id->vendor == VENDOR_INTEL) {
+	if(msr_platform_info_supported(info)) {
 		/* Refer links above
 		Table 35-12.  MSRs in Next Generation Intel Atom Processors Based on the Goldmont Microarchitecture
 		Table 35-13.  MSRs in Processors Based on Intel® Microarchitecture Code Name Nehalem
@@ -1069,8 +1283,10 @@ int msr_serialize_raw_data(struct msr_driver_t* handle, const char* filename)
 
 	/* Get cached decoded CPUID information */
 	id = get_cached_cpuid();
-	if (id->vendor == VENDOR_UNKNOWN)
+	if (id->vendor == VENDOR_UNKNOWN) {
+		fclose(f);
 		return cpuid_get_error();
+	}
 
 	/* Get CPU stock speed */
 	if (cpu_clock == 0)
@@ -1082,7 +1298,7 @@ int msr_serialize_raw_data(struct msr_driver_t* handle, const char* filename)
 		case VENDOR_HYGON:
 		case VENDOR_AMD:   msr = amd_msr;   break;
 		case VENDOR_INTEL: msr = intel_msr; break;
-		default: return cpuid_set_error(ERR_CPU_UNKN);
+		default: fclose(f); return cpuid_set_error(ERR_CPU_UNKN);
 	}
 
 	/* Print raw MSR values */
